// +-------------------------------------------------------------------------+
// |    index.js                   Main file for electron to init app env    |
// +-------------------------------------------------------------------------+

const { app, BrowserWindow, Menu, MenuItem, shell, contextBridge, ipcMain, screen} = require('electron')
const path = require('path')
const Badge = require('electron-windows-badge')
const {url} = require('url')
const settings = require('electron-settings')

const isMac = process.platform === 'darwin'
const isWin = process.platform === 'win32'

// Menu bar configuration
// ---------------------------------------------------------------------------

const menuTemplate = [
    // { role: 'appMenu' }
    ...(isMac ? [{
        label: app.name,
        submenu: [
            { role: 'about' },
            { type: 'separator' },
            { role: 'services' },
            { type: 'separator' },
            { role: 'hide' },
            { role: 'hideOthers' },
            { role: 'unhide' },
            { type: 'separator' },
            { role: 'quit' }
        ]
    }] : []),
    // { role: 'fileMenu' }
    {
        label: 'Datei',
        submenu: [
            {
                label: 'Neues Fenster',
                accelerator: isMac ? 'Cmd+N' : 'Ctrl+N',
                click: async () => {
                    createWindow()
                }
            },
            { role: 'close', label: 'Fenster schließen' },
            { role: 'quit', label: 'Beenden' }
        ]
    },
    // { role: 'editMenu' }
    {
        label: 'Bearbeiten',
        submenu: [
            { role: 'undo', label: 'Rückgängig' },
            { role: 'redo', label: 'Wiederherstellen' },
            { type: 'separator' },
            { role: 'cut', label: 'Ausschneiden' },
            { role: 'copy', label: 'Kopieren' },
            { role: 'paste', label: 'Einfügen' },
            ...(isMac ? [
                { role: 'pasteAndMatchStyle', label: 'Einfügen mit Stil' },
                { role: 'delete', label: 'Löschen' },
                { role: 'selectAll', label: 'Alles auswählen' },
                { type: 'separator' },
                {
                    label: 'Sprachausgabe',
                    submenu: [
                        { role: 'startSpeaking', label: 'Start' },
                        { role: 'stopSpeaking', label: 'Stop' }
                    ]
                }
            ] : [
                { role: 'delete', label: 'Löschen' },
                { type: 'separator' },
                { role: 'selectAll', label: 'Alles auswählen' }
            ])
        ]
    },
    // { role: 'viewMenu' }
    {
        label: 'Ansicht',
        submenu: [
            { role: 'reload', label: 'Neu laden' },
            { role: 'forceReload', label: 'Cache leeren' },
            // { role: 'toggleDevTools', label: 'Dev Tools' },
            { type: 'separator' },
            { role: 'resetZoom', label: 'Zoom zurücksetzen' },
            { role: 'zoomIn', label: 'Zoom vergrößern' },
            { role: 'zoomOut', label: 'Zoom verkleinern' },
            { type: 'separator' },
            { role: 'togglefullscreen', label: 'Vollbild' }
        ]
    },
    {
        role: 'help',
        label: 'Hilfe',
        submenu: [
            {
                label: 'Hilfe und Wissen',
                click: async () => {
                    const { shell } = require('electron')
                    await shell.openExternal('https://downrich.com/kb')
                }
            },
            {
                label: 'Downrich Webseite',
                click: async () => {
                    const { shell } = require('electron')
                    await shell.openExternal('https://downrich.com')
                }
            },
            { type: 'separator' },
            {
                label: 'Neoground GmbH',
                click: async () => {
                    const { shell } = require('electron')
                    await shell.openExternal('https://neoground.com')
                }
            }
        ]
    }
]

const menu = Menu.buildFromTemplate(menuTemplate)
Menu.setApplicationMenu(menu)

// Window
// ---------------------------------------------------------------------------

const createWindow = (url) => {

    const newWindowOffset = 20

    const savedWindowState = settings.get('windowState') || {
        width: 800,
        height: 600,
        x: -1,
        y: -1,
        maximized: false
    }

    const primaryDisplay = screen.getPrimaryDisplay()
    const { width, height } = primaryDisplay.workAreaSize

    // If small display (e.g. MacBooks) or saved as maximized, set maximized settings
    const shouldMaximize = (width <= 1280 && height <= 800) || savedWindowState.maximized || process.platform === 'linux'

    const win = new BrowserWindow({
        width: shouldMaximize ? width : width * 0.75,
        height: shouldMaximize ? height : height * 0.95,
        x: savedWindowState.x + newWindowOffset,
        y: savedWindowState.y + newWindowOffset,
        center: (savedWindowState.x < 0 && savedWindowState.y < 0),
        icon: path.join(__dirname, 'assets', 'logo_256.ico'),
        webPreferences: {
            nodeIntegration: true,
            contextIsolation: true,
            preload: path.resolve(__dirname, 'preload.js')
        }
    })

    // Maximize if needed
    if (shouldMaximize) {
        win.maximize();
    }

    new Badge(win, {
        font: '16px arial'
    })

    // Listen for window being closed, and save the window state
    win.on('close', () => {
        let bounds = win.getBounds()
        settings.set('windowState', {
            x: bounds.x,
            y: bounds.y,
            width: bounds.width,
            height: bounds.height,
            maximized: win.isMaximized()
        })
    })

    // Update handling
    const currentVersion = app.getVersion()
    if (settings.get('version') != currentVersion) {
        // Clear the cache
        win.webContents.session.clearCache().then(() => {
            console.log('Cache cleared for new version')
        })

        // Save the current version number
        settings.set('version', currentVersion)
    }

    if(!url) {
        url = 'https://downrich.com/downrich'
    }

    win.loadURL(url)
}

// Taskbar
// ---------------------------------------------------------------------------

if(isWin) {
    app.setUserTasks([
        {
            type: 'task',
            program: process.execPath,
            arguments: '--new-window',
            iconPath: process.execPath,
            iconIndex: 0,
            title: 'Neues Fenster',
            description: 'Öffne ein neues Fenster'
        }
    ])
}

// Create instance when ready
// ---------------------------------------------------------------------------

app.whenReady().then(() => {
    if (isWin) {
        app.setAppUserModelId(app.name);
    }

    createWindow()

    app.on('activate', () => {
        if (BrowserWindow.getAllWindows().length === 0) createWindow()
    })
})

// Close app when all windows are closed (except macOS)
// ---------------------------------------------------------------------------

app.on('window-all-closed', () => {
    if (process.platform !== 'darwin') {
        app.quit()
    }
})

// Open external links (with target="_blank") in browser
// ---------------------------------------------------------------------------

app.on('web-contents-created', (event, contents) => {

    contents.setWindowOpenHandler(({ url }) => {
        if ((url.includes('http') || url.includes('mailto')) && !url.includes('downrich')) {
            console.log('Opening in external: ' + url)
            shell.openExternal(url);
            return { action: 'deny' }
        }

        console.log('Link as usual')
        return { action: 'allow' }
    })
})

// IPC events (from preload.js)
// ---------------------------------------------------------------------------

ipcMain.on('newWindow', (event, args) => {
    createWindow(args)
})

ipcMain.on('newPopup', (event, args) => {
    const winwidth = 1280
    const winheight = 780

    const primaryDisplay = screen.getPrimaryDisplay()
    const { width, height } = primaryDisplay.workAreaSize

    let xoffset = width - winwidth
    let yoffset = height - winheight

    const win = new BrowserWindow({
        width: winwidth,
        height: winheight,
        x: xoffset,
        y: yoffset,
        icon: path.join(__dirname, 'assets', 'logo_256.ico'),
        webPreferences: {
            nodeIntegration: true,
            contextIsolation: true,
            preload: path.resolve(__dirname, 'preload.js')
        }
    })

    win.setAlwaysOnTop(true)
    win.focus()

    win.loadURL(args)
})

ipcMain.on('update-macos-badge', (event, args) => {
    if(args < 1) {
        app.dock.setBadge('')
    } else {
        app.dock.setBadge(String(args))
    }
})

ipcMain.on('setProgress', (event, args) => {
    let win = BrowserWindow.getAllWindows()[0]
    if(args < 0) {
        win.setProgressBar(-1)
    } else {
        win.setProgressBar(args / 100)
    }
})
